/*
 *	DatumProbe - main body of simple C program
 *
 *
 */

#include <limits.h>
#include <types.h>
#include <quickdraw.h>
#include <fonts.h>
#include <events.h>
#include <controls.h>
#include <windows.h>
#include <menus.h>
#include <textedit.h>
#include <dialogs.h>
#include <Menus.h>
#include <Devices.h>
#include <scrap.h>
#include <toolutils.h>
#include <memory.h>
#include <segload.h>
#include <files.h>
#include <osutils.h>
#include <diskinit.h>
#include <packages.h>
#include <traps.h>
#include <terminals.h>
#include <processes.h>

#include "Datum.h"
#include "DatumProbe.h"

#define k8Bit 42
#define k16Bit 43
#define k32Bit 36
#define kIOorRAM 34
	
static WindowPtr	gWindow;
static Rect			gRect;
static QDGlobals	qd;
BoardStatus			gStatus;
ClockData			gMainClock;
ClockData			gEventClock;
PCIConfigHeader		gConfigInfo;
DialogPtr			gProbeWin = nil;
DialogPtr			gDebugWin = nil;
Boolean				gDone = false;
short				gDataSize;
Boolean				gIOorMemory;
Boolean				gAutoUpdates;
unsigned long		gFakeData = 0xDEADBEEF;
unsigned long		gLastMainMajor = 0;

/*	Routines that utilize the DatumLib routines. This is the only place in the app
	that will access those routines. Please place all future routines here as well
*/

void	ReadMainTime()
{
	LatchMainClock();
	GetMainClock(&gMainClock);
}

void	ReadEventTime()
{
	LatchEventClock();
	GetEventClock(&gEventClock);
}

void	UnlockEventTime()
{
//	LatchEventClock();
//	LatchMainClock();
//	GetEventClock(&gEventClock);
//	GetMainClock(&gMainClock);
	ClearEventLock();
}

void	SetControlFlag(unsigned long	theBit)
{
	UInt32	theData;
	
	theData = GetBoardControl();
	theData = theData | theBit;
	SetBoardControl(theData);
}

void	ClrControlFlag(unsigned long	theBit)
{
	UInt32	theData;
	theData = GetBoardControl();
	theData = theData & (~theBit);
	SetBoardControl(theData);

}

Boolean	GetControlFlag(unsigned long	theBit)
{
	UInt32	theData;
	theData = GetBoardControl();
	theData = theData & theBit;
	return (theData != 0);
}

OSErr	GetClockConfigInfo()
{
	return noErr;
}

/* Done with interface routines */

void	MakeMenuBar()
{
	Handle	menuBar;
	
	menuBar = GetNewMBar(128);
	if (menuBar == nil) return;
	SetMenuBar(menuBar);
	DisposeHandle(menuBar);
	AppendResMenu(GetMenuHandle(128),'DRVR');	// add the desk accs
	DrawMenuBar();
}

pascal void ProbeDrawItem(WindowPtr theWindow, short itemNo)
{
	short	itemType;
	Handle	itemHandle;
	Rect	box;
	Str255	theText;
	short	oldFont;
	RGBColor	redColor, greenColor, oldColor;
	UInt32	theData;
	
	GetDialogItem(theWindow, itemNo, &itemType, &itemHandle, &box);
	SetPort(theWindow);
	oldFont = theWindow->txFont;
	TextFont(4);
	redColor.red = -1;
	redColor.green = 0;
	redColor.blue = 0;
	greenColor.green = -1;
	greenColor.red = 0;
	greenColor.blue = 0;
	GetForeColor(&oldColor);
	switch (itemNo) {
		case 13 :	// board status display
					EraseRect(&box);
				 	FrameRect(&box);
					MoveTo(box.left+3, box.top+12);
					if (gStatus.installed) DrawString("\pBoard Installed");
					else DrawString("\pBoard Not Installed");
					break;
		case  4 :	// Main clock Display
				 	FrameRect(&box);
					box.left = box.left+3;
					TextMode(srcCopy);
					MoveTo(box.left, box.top+14);
					DateString(gMainClock.ClockUpper+GoofyTimeOffset,true, theText,nil);
					DrawString(theText);
					DrawString("\p       ");
					TimeString(gMainClock.ClockUpper+GoofyTimeOffset,true, theText,nil);
					MoveTo(box.left, box.top+30);
					DrawString(theText);
					sprintf(theText,"\p.%06d  ",gMainClock.ClockLower & 0x000FFFFF);
					DrawString(theText);
					TextMode(srcOr);
					break;
		
		case  5 :	// Event clock Display
				 	FrameRect(&box);
					box.left = box.left+3;
					TextMode(srcCopy);
					MoveTo(box.left, box.top+14);
					DateString(gEventClock.ClockUpper+GoofyTimeOffset,true, theText,nil);
					DrawString(theText);
					DrawString("\p       ");
					TimeString(gEventClock.ClockUpper+GoofyTimeOffset,true, theText,nil);
					MoveTo(box.left, box.top+30);
					DrawString(theText);
					sprintf(theText,"\p.%06d  ",gEventClock.ClockLower & 0x000FFFFF);
					DrawString(theText);
					TextMode(srcOr);
					break;
		case 19	:
		case 20	:
		case 21	:
					theData = gMainClock.ClockLower;
					if (itemNo == 19) theData = theData & fFrequency;
					if (itemNo == 20) theData = theData & fPhase;
					if (itemNo == 21) theData = theData & fTracking;
					FrameOval(&box);
					InsetRect(&box,2,2);
					if (theData != 0) RGBForeColor(&redColor);
					else RGBForeColor(&greenColor);
					PaintOval(&box);
					RGBForeColor(&oldColor);
	}
	TextFont(oldFont);				
}

pascal void DrawDebugItem(WindowPtr theWindow, short itemNo)
{
	short	itemType;
	Handle	itemHandle;
	Rect	box;
	Str255	theText;
	short	oldFont;
	
	GetDialogItem(theWindow, itemNo, &itemType, &itemHandle, &box);
	SetPort(theWindow);
	EraseRect(&box);
	FrameRect(&box);
	oldFont = theWindow->txFont;
	TextFont(4);
	switch (itemNo) {
		case 1	:	sprintf(theText,"\pDevice & Status ID");
					break;
		case 2	:	sprintf(theText,"\pStatus/Command Reg");
					break;
		case 3	:	sprintf(theText,"\pClass Code/Rev");
					break;
		case 4	:	sprintf(theText,"\pDevice Info");
					break;
		case 5	:	sprintf(theText,"\pBase Addr 0");
					break;
		case 6	:	sprintf(theText,"\pBase Addr 1");
					break;
		case 7	:	sprintf(theText,"\pBase Addr 2");
					break;
		case 8	:	sprintf(theText,"\pBase Addr 3");
					break;
		case 9	:	sprintf(theText,"\pBase Addr 4");
					break;
		case 10	:	sprintf(theText,"\pBase Addr 5");
					break;
		case 11	:	sprintf(theText,"\pCIS Pointer");
					break;
		case 12	:	sprintf(theText,"\pSubSys Info");
					break;
		case 13	:	sprintf(theText,"\pROM Base Address");
					break;
		case 14	:	sprintf(theText,"\pReserved 1");
					break;
		case 15	:	sprintf(theText,"\pReserved 2");
					break;
		case 16	:	sprintf(theText,"\pInterupt Info");
					break;
		case 17 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.IDs);
					break;
		case 18 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.StatusCommand);
					break;
		case 19 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.ClassRev);
					break;
		case 20 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.DevInfo);
					break;
		case 21 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.BAR0);
					break;
		case 22 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.BAR1);
					break;
		case 23 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.BAR2);
					break;
		case 24 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.BAR3);
					break;
		case 25 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.BAR4);
					break;
		case 26 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.BAR5);
					break;
		case 27 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.CISPtr);
					break;
		case 28 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.SubsysIDs);
					break;
		case 29 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.ROMBase);
					break;
		case 30 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.Reserved1);
					break;
		case 31 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.Reserved2);
					break;
		case 32 :	sprintf(theText,"\p0x%08x        ",gConfigInfo.IntInfo);
					break;
	
	}
	MoveTo(box.left+3, box.top+12);
	DrawString(theText);
	TextFont(oldFont);				
}

void	SetDialogItemValue(DialogPtr theDlog, short theItem, Boolean On)
{
	short	itemType;
	Handle	itemHandle;
	Rect	box;
	
	GetDialogItem(theDlog, theItem, &itemType, &itemHandle, &box);
	if (On)
		SetControlValue((ControlHandle)itemHandle,1);
	if (!On)
		SetControlValue((ControlHandle)itemHandle,0);
}

short	GetDialogItemValue(DialogPtr theDlog, short theItem)
{
	short	itemType;
	Handle	itemHandle;
	Rect	box;
	
	GetDialogItem(theDlog, theItem, &itemType, &itemHandle, &box);
	return(GetControlValue((ControlHandle)itemHandle));
}

short	GetDialogItemString(DialogPtr theDlog, short theItem, Str255 theText)
{
	short	itemType;
	Handle	itemHandle;
	Rect	box;
	
	GetDialogItem(theDlog, theItem, &itemType, &itemHandle, &box);
	GetDialogItemText(itemHandle, theText);
	
	itemType = (short)theText[0];
	return(itemType);
}

void	SetDialogItemString(DialogPtr theDlog, short theItem, Str255 theText)
{
	short	itemType;
	Handle	itemHandle;
	Rect	box;
	
	GetDialogItem(theDlog, theItem, &itemType, &itemHandle, &box);
	SetDialogItemText(itemHandle, theText);
}

void	OpenProbeWindow()
{
	short	numItems, item;
	short	itemType;
	Handle	itemHandle;
	Rect	box;
	UserItemUPP	ProbeDrawProc = NewUserItemProc(ProbeDrawItem);
	
	gAutoUpdates = false;
	if (gProbeWin != nil)
		SelectWindow(gProbeWin);
	else {
		gProbeWin = GetNewDialog(128,nil, (WindowPtr)-1L);
		SetDialogItemValue(gProbeWin, 8, GetBoardControl()&fExternalTrig);
		SetDialogItemValue(gProbeWin, 9, GetBoardControl()&fFallingEdge);
		SetDialogItemValue(gProbeWin, 10, GetBoardControl()&fCaptureLock);
		SetDialogItemValue(gProbeWin, 11, GetBoardControl()&fEventsOn);
		numItems = CountDITL(gProbeWin);
		for (item=1; item <=numItems; item++) {
			GetDialogItem(gProbeWin, item, &itemType, &itemHandle, &box);
			if (itemType == userItem+itemDisable)
				SetDialogItem(gProbeWin, item, itemType, (Handle)ProbeDrawProc, &box);
			if (itemType == userItem)
				SetDialogItem(gProbeWin, item, itemType, (Handle)ProbeDrawProc, &box);
		}
		
	}
}

void	OpenDebugWindow()
{
	short	numItems, item;
	short	itemType;
	Handle	itemHandle;
	Rect	box;
	UserItemUPP	DebugDrawProc = NewUserItemProc(DrawDebugItem);
	

	GetCardConfigRec((char *)&gConfigInfo);
	
	if (gDebugWin != nil)
		SelectWindow(gDebugWin);
	else {
		gDebugWin = GetNewDialog(129,nil, (WindowPtr)-1L);
		numItems = CountDITL(gDebugWin);
		gDataSize = k32Bit;
		gIOorMemory = false;
		for (item=1; item <=numItems; item++) {
			GetDialogItem(gDebugWin, item, &itemType, &itemHandle, &box);
			if (itemType == userItem+itemDisable)
				SetDialogItem(gDebugWin, item, itemType, (Handle)DebugDrawProc, &box);
			if (itemType == userItem)
				SetDialogItem(gDebugWin, item, itemType, (Handle)DebugDrawProc, &box);
			if (item == k32Bit)	SetControlValue((ControlHandle)itemHandle,1);
		}
	}
}

void	CloseFrontWindow()
{
	DialogPtr	theWindow = FrontWindow();
	
	if (theWindow == nil) return;
	if (theWindow == gDebugWin) gDebugWin = nil;
	if (theWindow == gProbeWin) gProbeWin = nil;
	DisposeDialog(theWindow);
}

void	DoMenuCommand(long	menuResult)
{
	short	menuID, menuItem;
	Str255	itemName;
	short	daRefNum;
	
	menuID = HiWord(menuResult);
	menuItem = LoWord(menuResult);
	
	if (menuID == 128) {				// in Apple menu
		if (menuItem == 1) { daRefNum = Alert(130,nil);}
		else {
			GetMenuItemText(GetMenuHandle(128), menuItem, itemName);
			daRefNum = OpenDeskAcc(itemName);
		}
	}
	if (menuID == 129) {
		switch (menuItem) {
			case (1) :	OpenProbeWindow() ;break;			// open Datum probe window
			case (2) :  OpenDebugWindow() ;break;			// open Datum Debug window
			case (4) :  CloseFrontWindow();	break;		// close the current window
			case (6) :	gDone = true;
						break;
		}
	}
	HiliteMenu(0);					/* unhighlight what MenuSelect (or MenuKey) hilited */
}
	
void	ReDrawDItem(DialogPtr theDLog, short theItem)
{
	short	itemType;
	Handle	itemHandle;
	Rect	box;
	GrafPtr	oldPort;
	
	if (theDLog == nil) return;
	GetDialogItem(theDLog, theItem, &itemType, &itemHandle, &box);
	GetPort(&oldPort);
	SetPort(theDLog);
	InvalRect(&box);
	SetPort(oldPort);
}

void	HandleProbeHit(short itemHit)
{
	short	itemValue = GetDialogItemValue(gProbeWin, itemHit);
	switch (itemHit) {
		case 1	:	DisposeDialog(gProbeWin);
					gProbeWin = nil;
					break;
		case 2 : // latch the main time
					ReadMainTime();
					ReDrawDItem(gProbeWin, 4);
					break;
		case 3 : // latch and read the event time
					ReadEventTime();
					ReDrawDItem(gProbeWin, 5);
					break;
		case 7 : // Clear the lock on the event time
					UnlockEventTime();
					ReDrawDItem(gProbeWin, 5);
					ReDrawDItem(gProbeWin, 4);
					break;
		case 6 : // toggle the eventClock periodic update flag
					if (itemValue == 0) {
						gAutoUpdates = true;
						SetDialogItemValue(gProbeWin, itemHit, true);
					} else {
						gAutoUpdates = false;
						SetDialogItemValue(gProbeWin, itemHit, false);
					}						
					break;
		case 8 : // toggle the event capture flag
					if (itemValue == 0) {
						SetControlFlag(fExternalTrig);
						SetDialogItemValue(gProbeWin, itemHit, true);
					} else {
						ClrControlFlag(fExternalTrig);
						SetDialogItemValue(gProbeWin, itemHit, false);
					}						
					break;
		case 9 : // 
					if (itemValue == 0) {
						SetControlFlag(fFallingEdge);
						SetDialogItemValue(gProbeWin, itemHit, true);
					} else {
						ClrControlFlag(fFallingEdge);
						SetDialogItemValue(gProbeWin, itemHit, false);
					}						
					break;
		case 10 : // 
					if (itemValue == 0) {
						SetControlFlag(fCaptureLock);
						SetDialogItemValue(gProbeWin, itemHit, true);
					} else {
						ClrControlFlag(fCaptureLock);
						SetDialogItemValue(gProbeWin, itemHit, false);
					}						
					break;
		case 11 : // 
					if (itemValue == 0) {
						SetControlFlag(fEventsOn);
						SetDialogItemValue(gProbeWin, itemHit, true);
					} else {
						ClrControlFlag(fEventsOn);
						SetDialogItemValue(gProbeWin, itemHit, false);
					}						
					break;
		}
}

short	SetTheData(unsigned long *theAddr,unsigned long theData,short theStringSize) 
{
	unsigned short *theAddr16;
	unsigned char *theAddr8;

	if (theAddr == nil) theAddr = &gFakeData;
	theAddr16 = (unsigned short *)theAddr;
	theAddr8 = (unsigned char *)theAddr;
	
	if (gIOorMemory) { // we have to do an IO access
		theData = 0xFEDCBA98;
	} else {
		switch (gDataSize) {
			case k8Bit 	: if (theStringSize != 2) return(-1);
						  theAddr8[0] = theData>>24;
						  break;
			case k16Bit	: if (theStringSize != 4) return(-1);
						  theAddr16[0] = theData>>16;
						  break;
			case k32Bit	: if (theStringSize != 8) return(-1);
						  theAddr[0] = theData;
						  break;
		}
	}
	return noErr;
}

unsigned long	GetTheData(unsigned long *theAddr)
{
	unsigned short *theAddr16;
	unsigned char *theAddr8;
	unsigned long theData;
	
	if (theAddr == nil) theAddr = &gFakeData;
	theAddr16 = (unsigned short *)theAddr;
	theAddr8 = (unsigned char *)theAddr;
	theData = 0;
	if (gIOorMemory) { // we have to do an IO access
		theData = 0xFEDCBA98;
	} else {
		switch (gDataSize) {
			case k8Bit 	: theData = theAddr8[0];
						  theData = theData+(theData<<8)+(theData<<16)+(theData<<24);
						  break;
			case k16Bit	: theData = theAddr16[0];
						  theData = theData + (theData<<16);
						  break;
			case k32Bit	: theData = theAddr[0];
						  break;
		}
	}
	return theData;
}

void	HandleDebugHit( short itemHit)
{
	Str255	theText;
	short	theStringSize;
	unsigned long 	*theAddr;
	unsigned long 	theData32;
	
	switch (itemHit) {
		case 39	:	DisposeDialog(gDebugWin);
					gDebugWin = nil;
					break;
		case k8Bit :
					SetDialogItemValue(gDebugWin, k8Bit, true);
					SetDialogItemValue(gDebugWin, k16Bit, false);
					SetDialogItemValue(gDebugWin, k32Bit, false);
					gDataSize = k8Bit;
					break;
		case k16Bit :
					SetDialogItemValue(gDebugWin, k8Bit, false);
					SetDialogItemValue(gDebugWin, k16Bit, true);
					SetDialogItemValue(gDebugWin, k32Bit, false);
					gDataSize = k16Bit;
					break;
		case k32Bit :
					SetDialogItemValue(gDebugWin, k8Bit, false);
					SetDialogItemValue(gDebugWin, k16Bit, false);
					SetDialogItemValue(gDebugWin, k32Bit, true);
					gDataSize = k32Bit;
					break;
		case kIOorRAM :
					gIOorMemory = !gIOorMemory;
					if (gIOorMemory)
						SetDialogItemValue(gDebugWin, kIOorRAM, true);
					else
						SetDialogItemValue(gDebugWin, kIOorRAM, false);
					break;
		case 44	:
					theStringSize = GetDialogItemString(gDebugWin, 37, theText);
					if (theStringSize == 8) {
						theAddr = (unsigned long *)strtoul(theText+1, nil, 16);
						theData32 = GetTheData(theAddr);
						sprintf(theText, "\p%08x    ",theData32);
						if (gDataSize == k8Bit) theText[0] = 2;
						if (gDataSize == k16Bit) theText[0] = 4;
						SetDialogItemString(gDebugWin, 41, theText);
					} else Alert(131,nil);
					break;
		case 45	:	
					theStringSize = GetDialogItemString(gDebugWin, 37, theText);
					if (theStringSize == 8) {
						theAddr = (unsigned long *)strtoul(theText+1, nil, 16);
						theStringSize = GetDialogItemString(gDebugWin, 41, theText);
						theData32 = (unsigned long)strtoul(theText+1, nil, 16);
						if (SetTheData(theAddr, theData32, theStringSize) != noErr)
							Alert(132,nil);
					} else Alert(131,nil);
					break;
		}

}

void	HandleDLogEvent(EventRecord*	event)
{
	short	itemHit;
	DialogPtr	dWin;
	
	if (IsDialogEvent(event)) 
		if (DialogSelect(event, &dWin, &itemHit)) {
			if (dWin == gProbeWin) HandleProbeHit(itemHit);
			if (dWin == gDebugWin) HandleDebugHit(itemHit);
		}
}

void	ZoomEvent()
{
	UInt32	theIntStatus;
	if (gProbeWin == nil) return;
	
	ReadMainTime();
	ReDrawDItem(gProbeWin, 4);
	
	if (gAutoUpdates == false) return;
	theIntStatus = GetInteruptStatus();
	theIntStatus = theIntStatus & INT_EVENT;
	if (theIntStatus == 0) return;
	ResetInteruptStatus(theIntStatus);
	GetEventClock(&gEventClock);
	ReDrawDItem(gProbeWin, 5);
}

void	EventLoop()
{
	EventRecord	event;
	Boolean	gotEvent;
	short		part;
	WindowPtr	window;
	char		key;

	
	while (!gDone) {
		gotEvent = WaitNextEvent(everyEvent, &event, 6, nil);
		switch ( event.what ) {
			case nullEvent:
				HandleDLogEvent(&event);
				ZoomEvent();
				break;
			case mouseDown:
				part = FindWindow(event.where, &window);
				switch ( part ) {
					case inMenuBar:             /* process a mouse menu command (if any) */
						DoMenuCommand(MenuSelect(event.where));
						break;
					case inSysWindow:           /* let the system handle the mouseDown */
						SystemClick(&event, window);
						break;
					case inContent:
						if ( window != FrontWindow() ) {
							SelectWindow(window);
						}  else 
							HandleDLogEvent(&event);
						break;
					case inDrag:                /* pass screenBits.bounds to get all gDevices */
						DragWindow(window, event.where, &qd.screenBits.bounds);
						break;
					case inGoAway:
						if ( TrackGoAway(window, event.where) )
//							DoCloseWindow(window); /* we dont care if the user cancelled */
						break;
					case inGrow:
						break;
					case inZoomIn:
					case inZoomOut:
//					if ( TrackBox(window, event.where, part) )
//							DoZoomWindow(window, part);
						break;
				}
				break;
			case keyDown:
			case autoKey:                       /* check for menukey equivalents */
				key = event.message & charCodeMask;
				if ( event.modifiers & cmdKey ) {	/* Command key down */
					if ( event.what == keyDown ) {
						DoMenuCommand(MenuKey(key));
					}
				} else
					HandleDLogEvent(&event);
				break;
			case activateEvt:
				HandleDLogEvent(&event);
				break;
			case updateEvt:
				HandleDLogEvent(&event);
				break;
			}
		}
}

void main ()
{
	
	UInt32	tempData;
	
	ConnectionRtn();
	
// sets the output mode to 1Mhz
	tempData = GetBoardControl();
	tempData = tempData | fOutputFrequency;
	SetBoardControl(tempData);
	
	tempData = GetInteruptMask();
	tempData |= INT_EVENT;
	SetInteruptMask(tempData);
	
	InitGraf (&qd.thePort);
	InitFonts ();
	InitWindows ();
	InitCursor ();
	
	gStatus.installed = false;
	gStatus.status = 0;
	gStatus.control = 0;
	
	GetClockStatus(&gStatus);
	
	gMainClock.ClockUpper = 0x87654321;
	gMainClock.ClockLower = 0x12345678;

	gEventClock.ClockUpper = 0;
	gEventClock.ClockLower = 0;
	
	
	MakeMenuBar();
	
	EventLoop();
	ConnectionBroke();
}