/*	
				Datumlib.c
			
	Author:		Jim Mensch
	Date:		10-25-97
	
	Description:This file contains code for use with the Datum PCI card
				while they are universal they are specifically designed for PowerPC based
				Macintoshes
			
	Change History:
	
	10-25-97	First input into file
	
	
*/

// #define DebugLib	1

#include <limits.h>
#include <types.h>
#include <Devices.h>
#include <toolutils.h>
#include <memory.h>
#include <osutils.h>
#include <traps.h>
#include <timer.h>

#ifndef DebugLib
#include <NameRegistry.h>
#endif

#include "Datum.h"

PCIConfigHeader	ourHeader;
BoardStatus		ourStatus;
RegEntryID		NodeID;
OSErr			persistError;
Boolean			DebugMode = false;
#ifdef DebugLib
unsigned long	DebugData[64];
TMTask			myTask;
#endif

#ifdef	DebugLib
void	MyTimerTask(TMTaskPtr tmTaskPtr)
{
	DebugData[7] = INT_EVENT;
	PrimeTime((QElemPtr)tmTaskPtr,678);
	LatchEventClock();
}
#endif

OSErr	ConnectionRtn()
// InitBlockPtr myInitBlk)
{


#ifndef DebugLib

	UInt8	ConfigControl;
	RegEntryIter 			cookie; 
	RegEntryIterationOp 	iterOp; 
	Boolean 				done; 
	OSStatus 				err = noErr;

// Debugger();
// first find the Datum board somewhere in memory

	RegistryEntryIDInit(&NodeID);

	err = RegistryEntryIterateCreate(&cookie); 
	if (err != noErr) 
		return err;

	iterOp = kRegIterContinue; 
	err = RegistryEntrySearch(&cookie, iterOp, &NodeID, &done, 
							"name", DatumBoardName, strlen(DatumBoardName)+1);

	if (!done && (err == noErr)) 
	{ 
// wait here to see if we are done
	} else if (done) 
		err = done; 
	RegistryEntryIterateDispose(&cookie);
	 
	
	if (err != noErr) DebugMode = true;
// now load in the configuration record
	err = GetCardConfigRec((char *)(&ourHeader));
	if (err != noErr) {
		DebugMode = true;
		return noErr;
	}
// now enable the board for I/O and RAM writing thus "claiming" the board
	err = ExpMgrConfigReadByte(&NodeID, 4, &ConfigControl);
	ConfigControl = ConfigControl | 0x03;		// set RAM and IO enable
	err = ExpMgrConfigWriteByte(&NodeID, 4, ConfigControl);
#else
	GetCardConfigRec((char *)(&ourHeader));
	DebugMode = true;
	myTask.tmAddr = NewTimerProc(&MyTimerTask);
	InsTime((QElemPtr)&myTask);
	PrimeTime((QElemPtr)&myTask,687);
#endif
return ((OSErr)noErr);
}

void	ConnectionBroke()
{
#ifdef DebugLib
	RmvTime((QElemPtr)&myTask);
#endif
}

OSErr	GetClockStatus(BoardStatusPtr	theStatus)
{
	theStatus->installed = !DebugMode;
	theStatus->status = 1;
	return ((OSErr)noErr);
}

UInt32	ReadTheData(UInt32 *theAddr)
{
#ifndef DebugLib
 	return EndianSwap32Bit(*theAddr);
#else
	 	return (*theAddr);
#endif 
}

UInt32	xReadTheData(UInt32 *theAddr)
{
 	return (*theAddr);
}

void	WriteTheData(UInt32 *theAddr, UInt32 theData)
{
	
#ifndef DebugLib
	theAddr[0] = EndianSwap32Bit(theData);
#else
	theAddr[0] = theData;
#endif
}

/*
	UInt32	DatumRead(UInt32	offset);		// use the above defined offset fields
												// to access raw data from the clock
	UInt32	DatumWrite(UInt32	offset, UInt32	data);	// use to set data in the clock
	UInt32	SetEventControl(UInt32		data);	// sets up the event clock control register
*/	
	
void	ResetInteruptStatus(UInt32 theData)
{
	UInt32	theAddr;
	theAddr = ourHeader.BAR0+oInteruptStatus;
	WriteTheData((UInt32 *)theAddr, theData);
}

UInt32	GetInteruptStatus()
{
	UInt32	theAddr;
	theAddr = ourHeader.BAR0+oInteruptStatus;
	return ReadTheData((UInt32 *)theAddr);
}

UInt32	GetInteruptMask()
{
	UInt32	theAddr;
	theAddr = ourHeader.BAR0+oInteruptMask;
	return ReadTheData((UInt32 *)theAddr);
}

void	SetInteruptMask(UInt32 theData)
{
	UInt32	theAddr;
	theAddr = ourHeader.BAR0+oInteruptMask;
	WriteTheData((UInt32 *)theAddr, theData);
}

void	SetBoardControl(UInt32 theData)
{
	UInt32	theAddr;
	theAddr = ourHeader.BAR0+oControlRegister;
	WriteTheData((UInt32 *)theAddr, theData);
}

UInt32	GetBoardControl()
{
	UInt32	theAddr;
	theAddr = ourHeader.BAR0+oControlRegister;
	return ReadTheData((UInt32 *)theAddr);
}

UInt32	GetMainClock(ClockDataPtr	data)
{
	UInt32	theAddr;
	
	theAddr = ourHeader.BAR0+oMainClockMinor;
	data->ClockLower = ReadTheData((UInt32 *)theAddr);
	theAddr = ourHeader.BAR0+oMainClockMajor;
	data->ClockUpper = ReadTheData((UInt32 *)theAddr);
	return 0;
}

void	ClearEventLock ()
{
	UInt32	theAddr;
	
	theAddr = ourHeader.BAR0+oReleaseCaptureLock;
	theAddr = ReadTheData((UInt32 *)theAddr);
}

UInt32	GetEventClock(ClockDataPtr	data)
{
	UInt32	theAddr;
	
	theAddr = ourHeader.BAR0+oEventClockMinor;
	data->ClockLower = ReadTheData((UInt32 *)theAddr);
	theAddr = ourHeader.BAR0+oEventClockMajor;
	data->ClockUpper = ReadTheData((UInt32 *)theAddr);
	return 0;
}

#ifdef DebugLib
UInt32	GetCurrentTime()
{
	UInt32	theTime;
	
	GetDateTime(&theTime);
	theTime = theTime - GoofyTimeOffset;
	return theTime;
}
#endif

UInt32	LatchMainClock() 
{
	UInt32	theAddr;
	
	theAddr = ourHeader.BAR0 + oMainClockLatch; // start by creating the address for the event clock
	theAddr = ReadTheData((UInt32 *)theAddr);
	
#ifdef DebugLib
// here we fudge the time values in order to simulate the clock's
	theAddr = ourHeader.BAR0 + oMainClockMinor; // start by creating the address for the event clock
	WriteTheData((UInt32 *)theAddr,TickCount() + DebugData[0]);
	theAddr = ourHeader.BAR0 + oMainClockMajor; // start by creating the address for the event clock
	WriteTheData((UInt32 *)theAddr,GetCurrentTime());
	DebugData[0] = DebugData[0] + fTracking;
#endif

	return theAddr;
}	
	
UInt32	LatchEventClock()
{
	UInt32	theAddr;
	
	theAddr = ourHeader.BAR0 + oEventClockLatch; // start by creating the address for the event clock
	theAddr = ReadTheData((UInt32 *)theAddr);
	
#ifdef DebugLib
// here we fudge the time values in order to simulate the clock's
	theAddr = ourHeader.BAR0 + oEventClockMinor; // start by creating the address for the event clock
	WriteTheData((UInt32 *)theAddr,TickCount());
	theAddr = ourHeader.BAR0 + oEventClockMajor; // start by creating the address for the event clock
	WriteTheData((UInt32 *)theAddr,GetCurrentTime());
#endif

	return theAddr;
}
	

UInt32	GetCardConfigRec(Ptr	theDataBlock)
{
	short	i;
	OSErr	err = noErr;
	UInt32	*theDataPtr;
	
	theDataPtr = (UInt32 *) theDataBlock;
//	if (DebugMode) return noErr;
#ifndef DebugLib
	
	i = 0;
	while ((err == noErr) & (i<16))
	{
		err = ExpMgrConfigReadLong(&NodeID, i*4 , &theDataPtr[i]);
		i++;
	}
#else
	i = 0;
	while ((err == noErr) & (i<16))
	{
		theDataPtr[i] = i;
		if (i==4) {
			theDataPtr[i] = (UInt32)DebugData;
		}
		i++;
	}

#endif
	return (noErr);
}
	

